<?php

namespace Drupal\Tests\sparkpost\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * Test the mail system plugin.
 *
 * @group sparkpost
 */
class MailSystemTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'sparkpost',
  ];

  /**
   * Test the mime guesser part of the mail system.
   */
  public function testMimeGuesser() {
    // Set the sending to use a queue, so we can actually retrieve it and see
    // what it looks like.
    self::assertCount(0, $this->getQueueItems());
    $this->config('sparkpost.settings')
      ->set('async', TRUE)
      ->save();
    /** @var \Drupal\Core\Mail\MailManagerInterface $mail_manager */
    $mail_manager = $this->container->get('plugin.manager.mail');
    /** @var \Drupal\sparkpost\Plugin\Mail\SparkpostMail $mail_plugin */
    $mail_plugin = $mail_manager->createInstance('sparkpost_mail');
    $mail_plugin->mail([
      'to' => 'dummy@example.com',
      'id' => 'test_mail',
      'subject' => 'Test mail',
      'body' => 'This is a test',
      'attachments' => [
        $this->container->get('file_system')->realpath('core/misc/druplicon.png'),
      ],
    ]);
    $items = $this->getQueueItems();
    self::assertCount(1, $items);
    $message = $items[0];
    $sparkpost_message = $message->getSparkpostMessage();
    self::assertEquals($sparkpost_message["content"]["attachments"][0]["type"], 'image/png');
  }

  /**
   * Helper to get whats in the queue.
   *
   * @return \Drupal\sparkpost\MessageWrapper[]
   *   An array of queued messages.
   */
  protected function getQueueItems() : array {
    try {
      /** @var \Drupal\Core\Queue\QueueFactory $queue */
      $queue_factory = $this->container->get('queue');
      /** @var \Drupal\Core\Queue\QueueInterface $queue */
      $queue = $queue_factory->get('sparkpost_send');
      // Make sure the database table exists.
      $queue->createQueue();
      $items = [];
      while ($item = $queue->claimItem()) {
        $items[] = $item->data;
        $queue->deleteItem($item);
      }
      return $items;
    }
    catch (\Exception $e) {
      return [];
    }
  }

}
